/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 OW2 Consortium, http://www.ow2.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.framework.listener;

import java.lang.reflect.Array;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.atomic.AtomicInteger;

public class JBIListenerThreadFactory implements ThreadFactory {

    ThreadGroup group = null;

    final AtomicInteger threadNumber = new AtomicInteger(1);

    public JBIListenerThreadFactory(final String componentName) {

        final String jbiListenersThreadGroupName = componentName
                + " - JBI listeners";

        // We check if the threadGroup of the component JBI listener already
        // exists
        int nbThreadGroup = Thread.currentThread().getThreadGroup()
                .activeGroupCount();
        ThreadGroup[] threadGroups = (ThreadGroup[]) Array.newInstance(
                ThreadGroup.class, nbThreadGroup);
        nbThreadGroup = Thread.currentThread().getThreadGroup().enumerate(
                threadGroups);
        for (int i = 0; i < nbThreadGroup; i++) {
            if (jbiListenersThreadGroupName.equals(threadGroups[i].getName())) {
                this.group = threadGroups[i];
                break;
            }
        }

        if (this.group == null) {
            this.group = new ThreadGroup(Thread.currentThread()
                    .getThreadGroup(), jbiListenersThreadGroupName);
        }
    }

    public Thread newThread(Runnable r) {
        return new Thread(this.group, r, "JBI Listener thread #0x"
                + Integer.toString(threadNumber.getAndIncrement(), 16));
    }

}
