/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.component.framework.su;

import java.util.List;

import javax.jbi.management.DeploymentException;

import org.objectweb.petals.component.framework.PEtALSCFException;
import org.objectweb.petals.component.framework.bc.AbstractBindingComponent;
import org.objectweb.petals.component.framework.listener.ExternalListenerManager;
import org.objectweb.petals.jbi.descriptor.Consumes;

/**
 * Like SimpleServiceUnitManager but start and stop listeners for external
 * addresses.
 *
 * @author ofabre
 *
 */
public class BindingComponentServiceUnitManager extends
		ServiceUnitManager {

	private ExternalListenerManager externalListenerManager;

	public BindingComponentServiceUnitManager(
			AbstractBindingComponent bindingComponent) {
		super(bindingComponent);
		externalListenerManager = new ExternalListenerManager(component
				.getComponentConfiguration(), component,
				component.getContext(), component.getChannel(), component
						.getLogger());
	}

	@Override
	public void start(final String serviceUnitName) throws DeploymentException {
		super.start(serviceUnitName);
		try {
			externalListenerManager.startListening(serviceUnitName,
					getConsumesForSU(serviceUnitName));
		} catch (PEtALSCFException e) {
			String msg = "Failed to start SU : " + serviceUnitName;
			getLogger().severe(msg);
			throw new DeploymentException(msg, e);
		}
	}

	private List<Consumes> getConsumesForSU(String serviceUnitName)
			throws DeploymentException {
		ServiceUnitDataHandler handler = getServiceUnitDataHandlers().get(
				serviceUnitName);
		if (handler != null) {
			return handler.getConsumesList();
		} else {
			throw new DeploymentException("No data handler for su : "
					+ serviceUnitName + ". SU not deployed.");
		}
	}

	@Override
	public void stop(final String serviceUnitName) throws DeploymentException {
		try {
			externalListenerManager.stopListening(serviceUnitName);
		} catch (PEtALSCFException e) {
			String msg = "Failed to stop SU : " + serviceUnitName;
			getLogger().severe(msg);
			throw new DeploymentException(msg, e);
		}
		super.stop(serviceUnitName);
	}

}
